#include "BlockTypeGeneratorRule.h"

#include "Game.h"

BlockTypeGeneratorRule::BlockTypeGeneratorRule()
    : GeneratorRule(),
      blockType(0)
{}

Framework::Either<Block*, int> BlockTypeGeneratorRule::createBlock(
    int x, int y, int z, int dimensionId)
{
    return blockType;
}

void BlockTypeGeneratorRule::setBlockTypeId(int blockType)
{
    this->blockType = blockType;
}

int BlockTypeGeneratorRule::getBlockTypeId() const
{
    return blockType;
}

BlockTypeGeneratorRuleFactory::BlockTypeGeneratorRuleFactory()
    : GeneratorRuleFactory()
{}

BlockTypeGeneratorRule* BlockTypeGeneratorRuleFactory::createValue(
    Framework::JSON::JSONObject* zJson) const
{
    return new BlockTypeGeneratorRule();
}

BlockTypeGeneratorRule* BlockTypeGeneratorRuleFactory::fromJson(
    Framework::JSON::JSONObject* zJson) const
{
    BlockTypeGeneratorRule* result = GeneratorRuleFactory::fromJson(zJson);
    result->setBlockTypeId(Game::INSTANCE->getBlockTypeId(
        zJson->zValue("blockType")->asString()->getString()));
    return result;
}

Framework::JSON::JSONObject* BlockTypeGeneratorRuleFactory::toJsonObject(
    BlockTypeGeneratorRule* zObject) const
{
    Framework::JSON::JSONObject* result
        = GeneratorRuleFactory::toJsonObject(zObject);
    result->addValue("blockType",
        new Framework::JSON::JSONString(
            Game::INSTANCE->zBlockType(zObject->getBlockTypeId())->getName()));
    return result;
}

JSONObjectValidationBuilder* BlockTypeGeneratorRuleFactory::addToValidator(
    JSONObjectValidationBuilder* builder) const
{
    return GeneratorRuleFactory::addToValidator(
        builder->withRequiredAttribute("blockType",
            Game::INSTANCE->zTypeRegistry()->getValidator<Framework::Text>(
                BlockTypeNameFactory::TYPE_ID)));
}

const char* BlockTypeGeneratorRuleFactory::getTypeToken() const
{
    return "blockType";
}